import click
from pathlib import Path

import pynumbat

from pynumbat import current_location
from pynumbat import create_basic_config


@click.command()
@click.option('--path', '-p', default=current_location)
@click.option('--force', '-f', default=False)
@click.option('--virtual', '-v', default=True)
@click.option('--upload', '-u', default=False)
def main(path, force, virtual, upload):
    create_basic_config(Path(path), force, virtual, upload)


if __name__ == '__main__':
    main()


from pathlib import Path

current_location = Path.cwd()

README_CONTENT = """### README
"""

MAIN_CONTENT = ''''
def main(*args, **kwargs):
    """
    Main Function
    
    Args:
        None
    Returns:
        None
    """
    ...

if __name__ == '__main__':
    main()
'''

GITIGNORE_CONTENT = """# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
share/python-wheels/
*.egg-info/
.installed.cfg
*.egg
MANIFEST

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.nox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
*.py,cover
.hypothesis/
.pytest_cache/
cover/

# Translations
*.mo
*.pot

# Django stuff:
*.log
local_settings.py
db.sqlite3
db.sqlite3-journal

# Flask stuff:
instance/
.webassets-cache

# Scrapy stuff:
.scrapy

# Sphinx documentation
docs/_build/

# PyBuilder
.pybuilder/
target/

# Jupyter Notebook
.ipynb_checkpoints

# IPython
profile_default/
ipython_config.py

# pyenv
#   For a library or package, you might want to ignore these files since the code is
#   intended to run in multiple environments; otherwise, check them in:
# .python-version

# pipenv
#   According to pypa/pipenv#598, it is recommended to include Pipfile.lock in version control.
#   However, in case of collaboration, if having platform-specific dependencies or dependencies
#   having no cross-platform support, pipenv may install dependencies that don't work, or not
#   install all needed dependencies.
#Pipfile.lock

# PEP 582; used by e.g. github.com/David-OConnor/pyflow
__pypackages__/

# Celery stuff
celerybeat-schedule
celerybeat.pid

# SageMath parsed files
*.sage.py

# Environments
.env
.venv
env/
venv/
ENV/
env.bak/
venv.bak/

# Spyder project settings
.spyderproject
.spyproject

# Rope project settings
.ropeproject

# mkdocs documentation
/site

# mypy
.mypy_cache/
.dmypy.json
dmypy.json

# Pyre type checker
.pyre/

# pytype static type analyzer
.pytype/

# Cython debug symbols
cython_debug/
"""

SETUP_CFG = """# Inside of setup.cfg
[metadata]
description-file = README.md"""


SETUP_PY = """from pathlib import Path
from setuptools import setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

VERSION = '0.0.1'
DESCRIPTION = 'Description'
PACKAGE_NAME = 'Package Name'
AUTHOR = ''
EMAIL = ''
GITHUB_URL = ''

setup(
    name = PACKAGE_NAME,
    packages = [DESCRIPTION],
    version = VERSION,
    license='MIT',
    description = DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    author = AUTHOR,
    author_email = EMAIL,
    url = GITHUB_URL,
    keywords = [],
    install_requires=[ 
        'requests',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)"""


MIT_LICENSE = """MIT License
Copyright (c) 2018 YOUR NAME
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


import os
import sys

import logging 

from .common import MAIN_CONTENT
from .common import README_CONTENT
from .common import GITIGNORE_CONTENT

from .common import SETUP_PY
from .common import SETUP_CFG
from .common import MIT_LICENSE

from .decorators import check_if_folder_exists


def create_file(path, content):
    with open(path, 'w') as file:
        file.write(content)

    return True


@check_if_folder_exists
def create_app_folder(folder, path='./', force=False):
    
    current_path = path / folder
    current_test_path = current_path / 'tests'
    
    os.makedirs(current_path)
    os.makedirs(current_test_path)

    create_file(current_path / '__init__.py', '')
    create_file(current_path / '__main__.py', MAIN_CONTENT)
    create_file(current_path / 'config.py', '')
    
    create_file(current_test_path / 'tests.py', '')


def create_basic_config(path, force=False, virtual_env=True, upload=False):
    try:
        create_app_folder('app', path, force)
        create_basic_files(path)
        
        if virtual_env:
            create_virtual_env(path)

        if upload:
            create_pypi_files(path)

    except Exception as err:
        logging.error(err)


def create_basic_files(path):
    create_file(path / 'main.py', MAIN_CONTENT)
    create_file(path / 'README.md', README_CONTENT)
    create_file(path / '.gitignore', GITIGNORE_CONTENT)


def create_pypi_files(path):
    create_file(path / 'setup.py', SETUP_PY)
    create_file(path / 'setup.cfg', SETUP_CFG)
    create_file(path / 'LICENSE.txt', MIT_LICENSE)


def create_virtual_env(path, environment='env'):
    try:
        if python_3_6_or_greater():
            os.system(f"cd {path} && python -m venv {environment}")
            create_requirementes_txt(path, environment)
        else:
            logging.warning('In order to create the virtual environment Python >3.6 its necessary.')
            
    except Exception as err:
        return False


def create_requirementes_txt(path, environment):
    try:
        if sys.platform.startswith('win'):
            os.system(f"cd {path} && . {environment}\Scripts\activate && pip freeze > requirements.txt")
        else:
            os.system(f"cd {path} && . {environment}/bin/activate && pip freeze > requirements.txt")
    except Exception as err:
        logging.error(err)
        

def python_3_6_or_greater():
    if sys.version_info[0] == 3 and sys.version_info[1] >= 3:
        return True


class CreateFolderError(Exception):
    def __init__(self, folder):
        self.folder = folder
        self.message = f'Was not posible create "{folder}" folder.'
        
        super().__init__(self.message)


class CreateFileError(Exception):
    def __init__(self, file):
        self.file = file
        self.message = f'Was not posible create "{file}" file.'
        
        super().__init__(self.message)


from setuptools import setup, find_packages

from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

VERSION = '0.1.4'
DESCRIPTION = 'Python generator project'

setup(
    name = 'pynumbat',
    packages = ['pynumbat'],
    entry_points={
        "console_scripts":
            ["pynumbat=pynumbat.__main__:main"]
    },
    include_package_data=True,
    version = VERSION,
    license='MIT',
    description = DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    author = 'Eduardo Ismael Garcia Perez',
    author_email = 'eduardo78d@gmail.com',
    url = 'https://github.com/eduardogpg/pygenerate',
    keywords = ['Python Generate', 'Generate', 'Project'],
    install_requires=[ 
        'click',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)


import os
import shutil

from .exceptions import CreateFileError
from .exceptions import CreateFolderError


def raise_error(directory):
    if directory.is_file():
        raise CreateFileError(directory)
    
    elif directory.is_dir():
        raise CreateFolderError(directory)

    raise Exception('The task could not be completed.')


def delete_dir(directory):
    if directory.is_file():
        os.remove(directory)

    elif directory.is_dir():
        shutil.rmtree(directory)


def check_if_folder_exists(function):
    
    def wrapper(folder, path, force, *args, **kwargs):
        current_path = path / folder
            
        if current_path.exists() and force:
            delete_dir(current_path)

        response = function(folder, path, force, *args, **kwargs)
        return response
        
    return wrapper